using System;
using System.Collections.Generic;
using System.Linq;
using JetBrains.Annotations;
using Magify.Model;
using Magify.Types;
using UniRx;
using UnityEngine;
using EventType = Magify.Model.EventType;

namespace Magify
{
    internal class AnalyticsTracker : ILegacyMigrator, IInitializable, IDisposable
    {
        private static readonly MagifyLogger _logger = MagifyLogger.Get(LoggingScope.Analytics);

        private readonly CompositeDisposable _disposables = new();
        private readonly IDictionary<EventType, IAnalyticsEventHandler> _eventHandlers = new Dictionary<EventType, IAnalyticsEventHandler>();
        private readonly IServerApi _serverApi;
        private readonly GeneralPrefs _generalPrefs;
        private readonly string _storagePath;
        private readonly Subject<EventType> _onEventSent = new();
        private readonly Subject<(EventType, IAnalyticsEvent)> _onEventSentDetailed = new();

        public IObservable<EventType> OnEventSent => _onEventSent;
        public IObservable<(EventType Type, IAnalyticsEvent Event)> OnEventSentDetailed => _onEventSentDetailed;
        public LevelStateInfo LevelState { get; } = LevelStateInfo.Empty;

        public AnalyticsTracker(IServerApi serverApi, GeneralPrefs generalPrefs, string storagePath)
        {
            _serverApi = serverApi;
            _generalPrefs = generalPrefs;
            _storagePath = storagePath;
            CreateEventHandlers(_storagePath);
        }

        void ILegacyMigrator.Migrate(MigrationData data)
        {
            migrate(EventType.AppLaunch, data.AppLaunchEvents);
            migrate(EventType.SessionEvent, data.CustomSessionEvents);
            migrate(EventType.AppBackgroundEvent, data.AppBackgroundEvents);
            migrate(EventType.TrackInstall, data.TrackInstallEvents);
            migrate(EventType.Transaction, data.TransactionEvents);
            migrate(EventType.FirebaseMapping, data.FirebaseMappingEvents);
            migrate(EventType.MailingStatusMapping, data.MailingStatusMappingEvents);
            migrate(EventType.Impression, data.ImpressionEvents);
            migrate(EventType.ApplovinAdsImpression, data.ApplovinAdsImpressionEvents);
            migrate(EventType.IronSourceAdsImpression, data.IronSourceAdsImpressionEvents);
            migrate(EventType.ImpressionFail, data.ImpressionFailEvents);
            migrate(EventType.TrialActivation, data.TrialActivationEvents);
            migrate(EventType.PaidSubscriptionActivation, data.PaidSubscriptionActivationEvents);
            migrate(EventType.InApp, data.InAppEvents);
            migrate(EventType.Click, data.ClickEvents);
            migrate(EventType.AdsClick, data.AdsClickEvents);

            void migrate<T>(EventType eventType, [CanBeNull] IReadOnlyCollection<T> events) where T : class, IAnalyticsEvent
            {
                _logger.Log($"Trying to migrate {eventType} events");
                if (events == null)
                {
                    _logger.Log($"No {eventType} events to migrate");
                    return;
                }

                var eventHandler = (IAnalyticsEventHandler<T>)_eventHandlers[eventType];
                if (eventHandler.Storage.Exists())
                {
                    _logger.Log($"Storage for {eventType} events already exists. Skip migration");
                    return;
                }
                eventHandler.Storage.SaveEvents(events);
                _logger.Log($"Successfully migrated {events.Count} {eventType} events:\n{JsonFacade.SerializeObject(events)}");
            }
        }

        void IInitializable.Initialize()
        {
            _generalPrefs.FirebaseInstanceId
                .SkipLatestValueOnSubscribe()
                .Subscribe(TrackFirebaseMappingEvent)
                .AddTo(_disposables);

            foreach (var (_, value) in _eventHandlers)
            {
                value.Initialize();
            }
        }

        void IDisposable.Dispose()
        {
            _disposables.Dispose();
        }

        public void OnForeground()
        {
            foreach (var (_, value) in _eventHandlers)
            {
                value.OnForeground();
            }
        }

        public void OnBackground()
        {
            foreach (var (_, value) in _eventHandlers)
            {
                value.OnBackground();
            }
        }

        public void SetupAnalyticsConfig(AnalyticsConfiguration config)
        {
            foreach (var (_, value) in _eventHandlers)
            {
                value.SetupAnalyticsConfig(config);
            }
        }

        public void ResetAnalyticsConfig()
        {
            foreach (var (_, value) in _eventHandlers)
            {
                value.ResetAnalyticsConfig();
            }
        }

        #region Track Events

        public void TrackCustomSessionEvent(string eventName, Dictionary<string, object> parameters = null)
        {
            var customSessionEvent = new CustomSessionEvent(
                eventName,
                _generalPrefs.SessionId.Value,
                _generalPrefs.SubscriptionStatus.Value,
                _generalPrefs.InAppStatus.Value,
                parameters,
                LevelState,
                _generalPrefs.GlobalSessionCounter.Value);
            HandleEvent(EventType.SessionEvent, customSessionEvent);
        }

        public void TrackAppInstallEvent(HashSet<string> apps)
        {
            var @event = new TrackInstallEvent { InstalledApps = apps };
            HandleEvent(EventType.TrackInstall, @event);
        }

        public void TrackTransaction(Transaction transaction, IEnumerable<BonusInfo> bonuses, ProductInfo product = null, CampaignImpression impression = null)
        {
            var events = bonuses.Select(bonus => new TransactionEvent(
                transaction,
                bonus,
                _generalPrefs.GlobalSessionCounter.Value,
                impression,
                product,
                _generalPrefs.SubscriptionStatus.Value,
                _generalPrefs.InAppStatus.Value,
                LevelState,
                _generalPrefs.SessionId.Value
            ));
            HandleEvents(EventType.Transaction, new LinkedList<TransactionEvent>(events));
        }

        public void TrackFirebaseMappingEvent(string appInstanceId)
        {
            if (appInstanceId == null) return;
            var @event = new FirebaseMappingEvent { AppInstanceId = appInstanceId };
            HandleEvent(EventType.FirebaseMapping, @event);
        }

        public void TrackMailingStatusMappingEvent(string email)
        {
            var @event = new MailingStatusMappingEvent { Email = email };
            HandleEvent(EventType.MailingStatusMapping, @event);
        }

        public void TrackApplovinAdsImpression(CampaignImpression impression, ApplovinAdsImpression data)
        {
            var @event = new AdsImpressionEvent(
                impression,
                data,
                _generalPrefs.SubscriptionStatus.Value,
                _generalPrefs.InAppStatus.Value,
                LevelState,
                _generalPrefs.SessionId.Value);
            HandleEvent(EventType.ApplovinAdsImpression, @event);
        }

        public void TrackIronSourceAdsImpression(CampaignImpression impression, IronSourceAdsImpression data)
        {
            var @event = new AdsImpressionEvent(
                impression,
                data,
                _generalPrefs.SubscriptionStatus.Value,
                _generalPrefs.InAppStatus.Value,
                LevelState,
                _generalPrefs.SessionId.Value);
            HandleEvent(EventType.IronSourceAdsImpression, @event);
        }

        public void TrackPurchaseEvent(EventType type, CampaignImpression impression, PurchaseInfo purchase)
        {
            var analyticsEvent = new ProductPurchaseEvent(
                impression,
                purchase,
                _generalPrefs.SubscriptionStatus.Value,
                _generalPrefs.InAppStatus.Value,
                LevelState,
                _generalPrefs.SessionId.Value);
            HandleEvent(type, analyticsEvent);
        }

        public void TrackImpressionFailEvent(CampaignImpression impression, string reason)
        {
            var @event = new ImpressionFailEvent(
                impression,
                reason,
                _generalPrefs.SubscriptionStatus.Value,
                _generalPrefs.InAppStatus.Value,
                LevelState);
            HandleEvent(EventType.ImpressionFail, @event);
        }

        public void TrackAppLaunch()
        {
            var @event = new AppLaunchEvent
            {
                SubscriptionStatus = _generalPrefs.SubscriptionStatus.Value,
                InAppStatus = _generalPrefs.InAppStatus.Value
            };
            HandleEvent(EventType.AppLaunch, @event);
        }

        public void TrackAppBackgroundEvent(long openTime, long closeTime)
        {
            var @event = new AppBackgroundEvent
            {
                SessionId = _generalPrefs.SessionId.Value,
                OpenTimestamp = openTime.TimestampFromLongMills(),
                CloseTimestamp = closeTime.TimestampFromLongMills(),
            };
            HandleEvent(EventType.AppBackgroundEvent, @event);
        }

        public void TrackCommonEvents(EventType type, IReadOnlyList<CampaignImpression> impressions)
        {
            var models = new List<CampaignImpressionEvent>(impressions.Count);
            models.AddRange(impressions.Select(CreateCampaignImpressionEvent));
            HandleEvents(type, models);
        }

        public void TrackCommonEvent(EventType type, CampaignImpression impression)
        {
            var model = CreateCampaignImpressionEvent(impression);
            HandleEvent(type, model);
        }

        private CampaignImpressionEvent CreateCampaignImpressionEvent(CampaignImpression impression)
        {
            return new CampaignImpressionEvent(
                impression,
                _generalPrefs.SubscriptionStatus.Value,
                _generalPrefs.InAppStatus.Value,
                LevelState,
                _generalPrefs.SessionId.Value);
        }

        #endregion

        private void HandleEvent<T>(EventType eventType, T @event)
            where T : class, IAnalyticsEvent
        {
            ((IAnalyticsEventHandler<T>)_eventHandlers[eventType]).HandleEvent(@event);
            _onEventSent.OnNext(eventType);
            _onEventSentDetailed.OnNext((eventType, @event));
        }

        private void HandleEvents<T>(EventType eventType, IReadOnlyCollection<T> events)
            where T : class, IAnalyticsEvent
        {
            ((IAnalyticsEventHandler<T>)_eventHandlers[eventType]).HandleEvents(events);
            _onEventSent.OnNext(eventType);
            events.ForEach(@event => _onEventSentDetailed.OnNext((eventType, @event)));
        }

        private void CreateEventHandlers(string storagePath)
        {
            foreach (var eventTypeFlag in Enum.GetValues(typeof(EventType)).Cast<EventType>())
            {
                _eventHandlers.Add(eventTypeFlag, CreateEventHandleByEventType(eventTypeFlag, storagePath));
            }
        }

        private IAnalyticsEventHandler CreateEventHandleByEventType(EventType eventType, string storagePath)
        {
            return eventType switch
            {
                EventType.AppLaunch => new EventHandler<AppLaunchEvent>(eventType, _serverApi, storagePath),
                EventType.SessionEvent => new EventHandler<CustomSessionEvent>(eventType, _serverApi, storagePath),
                EventType.AppBackgroundEvent => new EventHandler<AppBackgroundEvent>(eventType, _serverApi, storagePath),
                EventType.TrackInstall => new EventHandler<TrackInstallEvent>(eventType, _serverApi, storagePath),
                EventType.Transaction => new EventHandler<TransactionEvent>(eventType, _serverApi, storagePath),
                EventType.FirebaseMapping => new EventHandler<FirebaseMappingEvent>(eventType, _serverApi, storagePath),
                EventType.MailingStatusMapping => new EventHandler<MailingStatusMappingEvent>(eventType, _serverApi, storagePath),
                EventType.ImpressionFail => new EventHandler<ImpressionFailEvent>(eventType, _serverApi, storagePath),
                EventType.Impression => new EventHandler<CampaignImpressionEvent>(eventType, _serverApi, storagePath),
                EventType.Click => new EventHandler<CampaignImpressionEvent>(eventType, _serverApi, storagePath),
                EventType.AdsClick => new EventHandler<CampaignImpressionEvent>(eventType, _serverApi, storagePath),
                EventType.ApplovinAdsImpression => new EventHandler<AdsImpressionEvent>(eventType, _serverApi, storagePath),
                EventType.IronSourceAdsImpression => new EventHandler<AdsImpressionEvent>(eventType, _serverApi, storagePath),
                EventType.InApp => new EventHandler<ProductPurchaseEvent>(eventType, _serverApi, storagePath),
                EventType.TrialActivation => new EventHandler<ProductPurchaseEvent>(eventType, _serverApi, storagePath),
                EventType.PaidSubscriptionActivation => new EventHandler<ProductPurchaseEvent>(eventType, _serverApi, storagePath),
                _ => throw new ArgumentOutOfRangeException(nameof(eventType), eventType, null)
            };
        }
    }
}