using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using Cysharp.Threading.Tasks;
using Magify.Model;
using UnityEngine;
using EventType = Magify.Model.EventType;

namespace Magify
{
    internal interface IAnalyticsEventHandler
    {
        void Initialize();
        void OnForeground();
        void OnBackground();
        void SetupAnalyticsConfig(AnalyticsConfiguration config);
        void ResetAnalyticsConfig();
    }

    internal interface IAnalyticsEventHandler<T> : IAnalyticsEventHandler
        where T : class, IAnalyticsEvent
    {
        AnalyticsEventStorage<T> Storage { get; }
        void HandleEvent(T analyticsEvent);
        void HandleEvents(IEnumerable<T> analyticsEvents);
    }

    internal class EventHandler<T> : IAnalyticsEventHandler<T>, IDisposable
        where T : class, IAnalyticsEvent
    {
        private static readonly MagifyLogger _logger = MagifyLogger.Get(LoggingScope.Analytics);
        private readonly IServerApi _serverApi;
        private readonly EventType _type;
        private readonly SyncTimer _syncTimer;
        private readonly AnalyticsConfiguration _defaultConfig;
        private AnalyticsConfiguration _currentConfig;
        private List<T> _events;
        private bool _isSendEventsInProgress;

        private int GroupMinSize => _currentConfig.GroupSize;
        private int MaxSyncInterval => _currentConfig.SyncIntervalInSeconds;
        public AnalyticsEventStorage<T> Storage { get; }

        public EventHandler(EventType type, IServerApi serverApi, string storagePath)
        {
            _serverApi = serverApi;
            _type = type;
            _syncTimer = new SyncTimer(() => PerformSync(true).Forget());
            Storage = new AnalyticsEventStorage<T>(type, storagePath);
            _currentConfig = _defaultConfig = new AnalyticsConfiguration
            {
                GroupSize = GetGroupMizSizeForEventType(type),
                SyncIntervalInSeconds = GetMaxSyncIntervalForEventType(type)
            };
        }

        public void Initialize()
        {
            InitWithStorage();
            PerformSync(true).Forget();
            _syncTimer.Start(MaxSyncInterval * 1000);
        }

        public void OnForeground()
        {
            _syncTimer.Start(MaxSyncInterval * 1000);
        }

        public void OnBackground()
        {
            _syncTimer.Stop();
            PerformSync(true).Forget();
        }

        public void HandleEvent(T @event)
        {
            _events.Add(@event);
            _logger.Log($"For {nameof(EventHandler<T>)} of type {typeof(T)} called {nameof(HandleEvent)} with event: \n{JsonFacade.SerializeObject(@event)}");
            StoreUnsentEvents();
            PerformSync().Forget();
        }

        public void HandleEvents(IEnumerable<T> events)
        {
            _events.AddRange(events);
            _logger.Log($"For {nameof(EventHandler<T>)} of type {typeof(T)} called {nameof(HandleEvents)} with events:\n{string.Join(",\n", _events.Select(JsonFacade.SerializeObject))}");
            StoreUnsentEvents();
            PerformSync().Forget();
        }

        public void SetupAnalyticsConfig(AnalyticsConfiguration config)
        {
            _currentConfig = config;
            OnConfigChanged();
        }

        public void ResetAnalyticsConfig()
        {
            _currentConfig = _defaultConfig;
            OnConfigChanged();
        }

        private void OnConfigChanged()
        {
            PerformSync().Forget();
            _syncTimer.Restart(MaxSyncInterval * 1000);
        }

        private void InitWithStorage()
        {
            _events = Storage.LoadEvents() ?? new List<T>();
            _logger.Log($"For {nameof(EventHandler<T>)} of type {typeof(T)} loaded events:\n{string.Join(",\n", _events.Select(JsonFacade.SerializeObject))}");
        }

        private void StoreUnsentEvents()
        {
            Storage.SaveEvents(_events);
            _logger.Log($"For {nameof(EventHandler<T>)} of type {typeof(T)} saved events:\n{string.Join(",\n", _events.Select(JsonFacade.SerializeObject))}");
        }

        private async UniTaskVoid PerformSync(bool isForced = false)
        {
            if (_events.Count == 0) return;
            if (_isSendEventsInProgress) return;

            if (isForced || _events.Count >= GroupMinSize)
            {
                _logger.Log($"For {nameof(EventHandler<T>)} of type {typeof(T)} called {nameof(PerformSync)} with events:\n{string.Join(",\n", _events.Select(JsonFacade.SerializeObject))}");
                _isSendEventsInProgress = true;
                var analyticsEvents = _events.ToArray();
                await TaskScheduler.SwitchToMainThread();
                var sendResult = await _serverApi.SendEvents(_type, analyticsEvents, CancellationToken.None);
                await UniTask.SwitchToMainThread();
                _isSendEventsInProgress = false;
                if (sendResult)
                {
                    _events.RemoveRange(0, analyticsEvents.Length);
                    StoreUnsentEvents();
                    PerformSync().Forget();
                }
                _syncTimer.Restart();
            }
        }

        private static int GetGroupMizSizeForEventType(EventType eventType)
        {
            switch (eventType)
            {
                case EventType.Impression:
                case EventType.ApplovinAdsImpression:
                case EventType.IronSourceAdsImpression:
                case EventType.SessionEvent:
                    return 10;
                default:
                    return 1;
            }
        }

        private static int GetMaxSyncIntervalForEventType(EventType eventType)
        {
            switch (eventType)
            {
                case EventType.Impression:
                    return 10;
                case EventType.ApplovinAdsImpression:
                case EventType.IronSourceAdsImpression:
                case EventType.SessionEvent:
                    return 60;
                default:
                    return 0;
            }
        }

        public void Dispose()
        {
            _syncTimer?.Dispose();
        }
    }
}