#if UNITY_EDITOR || UNITY_ANDROID
using System;
using System.IO;
using JetBrains.Annotations;
using Magify.Model;
using UniRx;
using UnityEngine;

namespace Magify
{
    internal partial class PlatformAndroid : PlatformAPI, IInitializable
    {
        internal const string LegacyCountersFileName = "legacy-counters.bin";

        private readonly CompositeDisposable _disposable = new();
        private readonly Subject<object> _onBackground = new();
        private readonly Subject<object> _onForeground = new();
        private readonly Subject<LocaleData> _onLocaleChanged = new();
        private readonly Subject<string> _onNewPackageInstalled = new();
        private readonly AndroidJavaObject _magifyClient;
        private readonly AndroidLegacyCountersStorage _legacyCountersStorage;

        public override RuntimePlatform RuntimePlatform => RuntimePlatform.Android;

        /// <remarks>
        /// Important: switching to main thread may cause foreground/background events to be skipped when enter background/overlapping with native UI.
        /// </remarks>
        public override IObservable<object> OnBackground => _onBackground;
        public override IObservable<object> OnForeground => _onForeground;
        public override IObservable<LocaleData> OnLocaleChanged => _onLocaleChanged.ObserveOnMainThread();
        public override IObservable<string> OnNewPackageInstalled => _onNewPackageInstalled.ObserveOnMainThread();

        public const int DensityMedium = 160;
        public const int DensityHigh = 240;
        public const int DensityXhigh = 320;
        public const int DensityXxhigh = 480;
        public const int DensityXxxhigh = 640;

        public override string DeviceName => $"{DeviceType}-{Density}";
        public override string DeviceVersion => GetSDKInt();

        private string DeviceType => _magifyClient.Call<bool>("isTablet") ? "tablet" : "phone";

        private string Density => Screen.dpi switch
        {
            >= DensityXxxhigh => "xxxhdpi",
            >= DensityXxhigh => "xxhdpi",
            >= DensityXhigh => "xhdpi",
            >= DensityHigh => "hdpi",
            >= DensityMedium => "mdpi",
            _ => "ldpi"
        };

        public PlatformAndroid(string storagePath)
        {
            _legacyCountersStorage = AndroidLegacyCountersStorage.Create(Path.Combine(storagePath, LegacyCountersFileName)).AddTo(_disposable);

            using var clsClient = new AndroidJavaClass("com.magify.androidsdk.MagifyClient");
            _magifyClient = clsClient.CallStatic<AndroidJavaObject>("getInstance").AddTo(_disposable);

            using var unityPlayer = new AndroidJavaClass("com.unity3d.player.UnityPlayer");
            using var currentActivity = unityPlayer.GetStatic<AndroidJavaObject>("currentActivity");
            using var androidApp = currentActivity.Call<AndroidJavaObject>("getApplication");
            _magifyClient.Call("setup", androidApp);
        }

        void IInitializable.Initialize()
        {
            _magifyClient.Call("setForegroundStateChangeCallback", new ForegroundStateChangedCallback(_onForeground, _onBackground));
            _magifyClient.Call("setLocaleChangeCallback", new LocaleChangedCallback(_onLocaleChanged));
            _magifyClient.Call("setNewPackageInstallCallback", new NewPackageInstalledCallback(_onNewPackageInstalled));
        }

        public override void AddPlatformSpecificData(AuthTokenPayload authToken, GeneralPrefs prefs, bool isGeoIpEnabled)
        {
            base.AddPlatformSpecificData(authToken, prefs, isGeoIpEnabled);
            // Pass null if isGeoIpEnabled == true (absent parameter will be treated by server as true)
            // In order to hide this parameter for sake of anti-fraud
            authToken.IsGeoIPEnabled = isGeoIpEnabled ? null : false;
            authToken.GpsAdId = prefs.GpsAdId.Value;
        }

        private static string GetSDKInt()
        {
            using var version = new AndroidJavaClass("android.os.Build$VERSION");
            return version.GetStatic<int>("SDK_INT").ToString();
        }

        public override LocaleData GetCurrentLocale()
        {
            var countryCode = _magifyClient.Call<string>("getCountryCode");
            var language = _magifyClient.Call<string>("getLanguageWithRegion");
            return new LocaleData(countryCode, language);
        }

        public override void Dispose()
        {
            _magifyClient.Call("dispose");
            _disposable.Dispose();
            base.Dispose();
        }

        public override void TweakUserLocale(string languageTag)
        {
            _magifyClient.Call("tweakUserLocale", languageTag);
        }

        public override void TrackPromotedApplication(string bundleId)
        {
            // Do nothing
        }

        public override bool IsApplicationInstalled(string bundleId)
        {
            return _magifyClient.Call<bool>("isPackageInstalled", bundleId);
        }

        public override bool IsApplicationInstalled(PromotedApplication application)
        {
            return IsApplicationInstalled(application?.BundleId);
        }

        public override string GetGpsAdId()
        {
            return CustomGpsAdId ?? _magifyClient.Call<string>("getGpsAdId");
        }

        private class LocaleChangedCallback : AndroidJavaProxy
        {
            private static readonly MagifyLogger _logger = MagifyLogger.Get();
            private readonly Subject<LocaleData> _localeChangedHandler;

            public LocaleChangedCallback(Subject<LocaleData> localeChangedHandler) : base("com.magify.androidsdk.callback.OnLocaleChangeCallback")
            {
                _localeChangedHandler = localeChangedHandler;
            }

            [UsedImplicitly]
            // ReSharper disable once InconsistentNaming
            public void onChange(string countryCode, string languageWithRegion)
            {
                _logger.Log($"Native locale changed callback received with {nameof(countryCode)}: {countryCode}, {nameof(languageWithRegion)}: {languageWithRegion}");
                var localeData = new LocaleData(countryCode, languageWithRegion);
                _localeChangedHandler?.OnNext(localeData);
            }
        }

        private class ForegroundStateChangedCallback : AndroidJavaProxy
        {
            private readonly Subject<object> _foregroundHandler;
            private readonly Subject<object> _backgroundHandler;

            public ForegroundStateChangedCallback(Subject<object> foregroundHandler, Subject<object> backgroundHandler) : base("com.magify.androidsdk.callback.OnForegroundStateChangeCallback")
            {
                _foregroundHandler = foregroundHandler;
                _backgroundHandler = backgroundHandler;
            }

            [UsedImplicitly]
            // ReSharper disable once InconsistentNaming
            public void onChange(bool isForeground)
            {
                if (isForeground)
                {
                    _foregroundHandler.OnNext(this);
                }
                else
                {
                    _backgroundHandler.OnNext(this);
                }
            }
        }

        private class NewPackageInstalledCallback : AndroidJavaProxy
        {
            private readonly Subject<string> _packageInstalledHandler;

            public NewPackageInstalledCallback(Subject<string> packageInstalledHandler) : base("com.magify.androidsdk.callback.OnNewPackageInstallCallback")
            {
                _packageInstalledHandler = packageInstalledHandler;
            }

            [UsedImplicitly]
            // ReSharper disable once InconsistentNaming
            public void onNewInstall(string packageName)
            {
                _packageInstalledHandler.OnNext(packageName);
            }
        }
    }
}
#endif