﻿using System;
using System.Collections.Generic;
using System.Text;
using Cysharp.Threading.Tasks;
using Magify.Model;
using UniRx;
using UnityEngine;
using UnityEngine.Pool;

namespace Magify
{
    internal class PurchaseTracker : ILegacyMigrator, IInitializable, ICancelable
    {
        private const int SyncTimeMills = 60 * 1000;
        private static readonly MagifyLogger _logger = MagifyLogger.Get(LoggingScope.Purchases);

        private readonly IServerApi _serverApi;
        private readonly PlatformAPI _platform;
        private readonly PurchasesStorage _storage;
        private readonly List<PurchaseRecord> _uploadingPurchaseRecords = new();
        private readonly SyncTimer _syncTimer;
        private List<PurchaseRecord> _unsentPurchaseRecords;
        private CompositeDisposable _disposables = GenericPool<CompositeDisposable>.Get();

        public bool IsDisposed => _disposables == null || _disposables.IsDisposed;

        public PurchaseTracker(IServerApi serverApi, PlatformAPI platform, string storagePath)
        {
            _serverApi = serverApi;
            _platform = platform;
            _storage = new PurchasesStorage(storagePath);
            _syncTimer = new SyncTimer(SendRecords).AddTo(_disposables);
        }

        public void Initialize()
        {
            ThrowIfDisposed();
            _unsentPurchaseRecords = _storage.LoadRecords();
            _syncTimer.Start(SyncTimeMills);
            _platform.OnForeground
                .Subscribe(_ => OnForeground())
                .AddTo(_disposables);
            _platform.OnBackground
                .Subscribe(_ => OnBackground())
                .AddTo(_disposables);
            SendRecords();
        }

        public void ValidateInApp(string productId, string transactionId = null, string purchaseToken = null, string receipt = null)
        {
            ValidatePurchase(productId, false, transactionId, purchaseToken, receipt);
        }

        public void ValidateSubscription(string productId, string transactionId = null, string purchaseToken = null, string receipt = null)
        {
            ValidatePurchase(productId, true, transactionId, purchaseToken, receipt);
        }

        public void ValidatePurchase(string productId, bool isSubscription, string transactionId = null, string purchaseToken = null, string receipt = null)
        {
            ThrowIfDisposed();
            var purchaseRecord = new PurchaseRecord
            {
                ProductId = productId,
#if UNITY_ANDROID
                IsSubscription = isSubscription,
                TransactionId = transactionId,
                PurchaseToken = purchaseToken,
#elif UNITY_IOS
                Receipt = receipt,
                DeviceId = _platform.GetEncodedDeviceIdentifier(),
                BuildNumber = _platform.BuildNumber,
#endif
            };
            AddUnsentPurchaseRecord(purchaseRecord);
            SendRecords();
            _syncTimer.Restart();
        }

        private void SendRecords()
        {
            foreach (var purchaseRecord in _unsentPurchaseRecords)
            {
                SendRecord(purchaseRecord).Forget();
            }
        }

        private async UniTaskVoid SendRecord(PurchaseRecord purchaseRecord)
        {
            if (_uploadingPurchaseRecords.Contains(purchaseRecord)) return;

            try
            {
                _uploadingPurchaseRecords.Add(purchaseRecord);
                var purchaseVerificationResult = await _serverApi.VerifyPurchase(purchaseRecord, _disposables.GetOrCreateToken());
                await UniTask.SwitchToMainThread();
                switch (purchaseVerificationResult)
                {
                    case PurchaseVerificationResult.Fail:
                        break;

                    case PurchaseVerificationResult.Success:
                    case PurchaseVerificationResult.DoesntSupport: // In the case if app is not configured in purchases validator we have to remove not verified purchase from the local storage
                        RemoveSentPurchaseRecord(purchaseRecord);
                        break;
                }
            }
            finally
            {
                if (!IsDisposed)
                {
                    _uploadingPurchaseRecords.Remove(purchaseRecord);
                    _syncTimer.Restart();
                }
            }
        }

        private void AddUnsentPurchaseRecord(PurchaseRecord purchaseRecord)
        {
            _unsentPurchaseRecords.Add(purchaseRecord);
            _storage.SaveRecords(_unsentPurchaseRecords);
        }

        private void RemoveSentPurchaseRecord(PurchaseRecord purchaseRecord)
        {
            _unsentPurchaseRecords.Remove(purchaseRecord);
            _storage.SaveRecords(_unsentPurchaseRecords);
        }

        private void OnForeground()
        {
            _syncTimer.Start(SyncTimeMills);
        }

        private void OnBackground()
        {
            _syncTimer.Stop();
            _storage.SaveRecords(_unsentPurchaseRecords);
        }

        private void ThrowIfDisposed()
        {
            if (IsDisposed)
            {
                throw new ObjectDisposedException(nameof(PurchaseTracker));
            }
        }

        public void Dispose()
        {
            ThrowIfDisposed();
            _disposables.Clear();
            GenericPool<CompositeDisposable>.Release(_disposables);
            _disposables = default;
            _uploadingPurchaseRecords.Clear();
        }

        public void Migrate(MigrationData data)
        {
            _logger.Log("Try to migrate purchases from native");
            if (_storage.Exists())
            {
                _logger.Log("Saved purchases already exists. Skip migration.");
                return;
            }
            if (data.PurchasedRecords == null)
            {
                _logger.Log("No purchases to migration data. Skip migration.");
                return;
            }
            _storage.SaveRecords(data.PurchasedRecords!);
            _logger.Log($"Purchases successfully migrated in folder: {_storage.RootFolderPath}");
        }
    }
}