﻿using System;
using System.Collections.Generic;
using System.Linq;
using JetBrains.Annotations;
using UniRx;

namespace Magify
{
    internal class SubsystemsCollection : ICancelable
    {
        private readonly List<object> _subsystems = new();

        public bool IsDisposed { get; private set; }

        public void Register(object instance)
        {
            ThrowIfDisposed();
            _subsystems.Add(instance);
        }

        public T Register<T>([NotNull] T instance)
        {
            Register((object)instance);
            return instance;
        }

        public T Register<T>([NotNull] Func<T> creator)
        {
            return Register(creator());
        }

        public void Clear()
        {
            ThrowIfDisposed();
            foreach (var subsystem in _subsystems)
            {
                DisposeSubsystem(subsystem);
            }
            _subsystems.Clear();
        }

        public bool Remove(object subsystem)
        {
            ThrowIfDisposed();
            if (_subsystems.Remove(subsystem))
            {
                DisposeSubsystem(subsystem);
                return true;
            }
            return false;
        }

        public int RemoveAll<T>()
        {
            ThrowIfDisposed();
            var removed = 0;
            for (var i = 0; i < _subsystems.Count;)
            {
                var subsystem = _subsystems[i];
                if (subsystem is T typed)
                {
                    DisposeSubsystem(subsystem);
                    _subsystems.RemoveAt(i);
                    removed++;
                }
                else
                {
                    i++;
                }
            }
            return removed;
        }

        public IEnumerable<T> OfType<T>()
        {
            ThrowIfDisposed();
            return _subsystems.OfType<T>();
        }

        public void ForEach<T>([NotNull] Action<T> action)
        {
            ThrowIfDisposed();
            _subsystems.OfType<T>().ForEach(action);
        }

        void IDisposable.Dispose()
        {
            if (IsDisposed)
            {
                return;
            }
            Clear();
            IsDisposed = true;
        }

        private void DisposeSubsystem(object subsystem)
        {
            switch (subsystem)
            {
                case ICancelable { IsDisposed: false } cancelable:
                    cancelable.Dispose();
                    break;
                case IDisposable disposable:
                    disposable.Dispose();
                    break;
            }
        }

        private void ThrowIfDisposed()
        {
            if (IsDisposed)
            {
                throw new ObjectDisposedException(nameof(SubsystemsCollection));
            }
        }
    }

    internal static class SubsystemsCollectionExtensions
    {
        public static T AddTo<T>(this T instance, SubsystemsCollection subsystemsCollection)
        {
            return subsystemsCollection.Register(instance);
        }
    }
}